"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectTestingLibraryUtils = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const node_utils_1 = require("./node-utils");
const utils_1 = require("./utils");
const DEFAULT_FILENAME_PATTERN = '^.*\\.(test|spec)\\.[jt]sx?$';
const FIRE_EVENT_NAME = 'fireEvent';
function detectTestingLibraryUtils(ruleCreate) {
    return (context, optionsWithDefault) => {
        var _a;
        let importedTestingLibraryNode = null;
        let importedCustomModuleNode = null;
        const customModule = context.settings['testing-library/module'];
        const filenamePattern = (_a = context.settings['testing-library/filename-pattern']) !== null && _a !== void 0 ? _a : DEFAULT_FILENAME_PATTERN;
        const isAggressiveReportingEnabled = () => !customModule;
        const getTestingLibraryImportNode = () => {
            return importedTestingLibraryNode;
        };
        const getCustomModuleImportNode = () => {
            return importedCustomModuleNode;
        };
        const getTestingLibraryImportName = () => {
            return node_utils_1.getImportModuleName(importedTestingLibraryNode);
        };
        const getCustomModuleImportName = () => {
            return node_utils_1.getImportModuleName(importedCustomModuleNode);
        };
        const isTestingLibraryImported = () => {
            if (isAggressiveReportingEnabled()) {
                return true;
            }
            return !!importedTestingLibraryNode || !!importedCustomModuleNode;
        };
        const isValidFilename = () => {
            const fileName = context.getFilename();
            return !!fileName.match(filenamePattern);
        };
        const isGetByQuery = (node) => {
            return /^get(All)?By.+$/.test(node.name);
        };
        const isQueryByQuery = (node) => {
            return /^query(All)?By.+$/.test(node.name);
        };
        const isFindByQuery = (node) => {
            return /^find(All)?By.+$/.test(node.name);
        };
        const isSyncQuery = (node) => {
            return isGetByQuery(node) || isQueryByQuery(node);
        };
        const isAsyncQuery = (node) => {
            return isFindByQuery(node);
        };
        const isCustomQuery = (node) => {
            return ((isSyncQuery(node) || isAsyncQuery(node)) &&
                !utils_1.ALL_QUERIES_COMBINATIONS.includes(node.name));
        };
        const isAsyncUtil = (node) => {
            return utils_1.ASYNC_UTILS.includes(node.name);
        };
        const isFireEventMethod = (node) => {
            const fireEventUtil = findImportedUtilSpecifier(FIRE_EVENT_NAME);
            let fireEventUtilName;
            if (fireEventUtil) {
                fireEventUtilName = experimental_utils_1.ASTUtils.isIdentifier(fireEventUtil)
                    ? fireEventUtil.name
                    : fireEventUtil.local.name;
            }
            else if (isAggressiveReportingEnabled()) {
                fireEventUtilName = FIRE_EVENT_NAME;
            }
            if (!fireEventUtilName) {
                return false;
            }
            const parentMemberExpression = node_utils_1.isMemberExpression(node.parent) ? node.parent : undefined;
            if (!parentMemberExpression) {
                return false;
            }
            if ([fireEventUtilName, FIRE_EVENT_NAME].includes(node.name) ||
                (experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object) &&
                    parentMemberExpression.object.name === node.name)) {
                return false;
            }
            const regularCall = experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object) &&
                parentMemberExpression.object.name === fireEventUtilName;
            const wildcardCall = node_utils_1.isMemberExpression(parentMemberExpression.object) &&
                experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object.object) &&
                parentMemberExpression.object.object.name === fireEventUtilName &&
                experimental_utils_1.ASTUtils.isIdentifier(parentMemberExpression.object.property) &&
                parentMemberExpression.object.property.name === FIRE_EVENT_NAME;
            return regularCall || wildcardCall;
        };
        const isPresenceAssert = (node) => {
            const { matcher, isNegated } = node_utils_1.getAssertNodeInfo(node);
            if (!matcher) {
                return false;
            }
            return isNegated
                ? utils_1.ABSENCE_MATCHERS.includes(matcher)
                : utils_1.PRESENCE_MATCHERS.includes(matcher);
        };
        const isAbsenceAssert = (node) => {
            const { matcher, isNegated } = node_utils_1.getAssertNodeInfo(node);
            if (!matcher) {
                return false;
            }
            return isNegated
                ? utils_1.PRESENCE_MATCHERS.includes(matcher)
                : utils_1.ABSENCE_MATCHERS.includes(matcher);
        };
        const findImportedUtilSpecifier = (specifierName) => {
            var _a;
            const node = (_a = getCustomModuleImportNode()) !== null && _a !== void 0 ? _a : getTestingLibraryImportNode();
            if (!node) {
                return null;
            }
            if (node_utils_1.isImportDeclaration(node)) {
                const namedExport = node.specifiers.find((n) => node_utils_1.isImportSpecifier(n) && n.imported.name === specifierName);
                if (namedExport) {
                    return namedExport;
                }
                return node.specifiers.find((n) => node_utils_1.isImportNamespaceSpecifier(n));
            }
            else {
                const requireNode = node.parent;
                if (experimental_utils_1.ASTUtils.isIdentifier(requireNode.id)) {
                    return requireNode.id;
                }
                const destructuring = requireNode.id;
                const property = destructuring.properties.find((n) => node_utils_1.isProperty(n) &&
                    experimental_utils_1.ASTUtils.isIdentifier(n.key) &&
                    n.key.name === specifierName);
                return property.key;
            }
        };
        const canReportErrors = () => {
            return isTestingLibraryImported() && isValidFilename();
        };
        const isNodeComingFromTestingLibrary = (node) => {
            let identifierName;
            if (experimental_utils_1.ASTUtils.isIdentifier(node)) {
                identifierName = node.name;
            }
            else if (experimental_utils_1.ASTUtils.isIdentifier(node.object)) {
                identifierName = node.object.name;
            }
            if (!identifierName) {
                return;
            }
            return !!findImportedUtilSpecifier(identifierName);
        };
        const helpers = {
            getTestingLibraryImportNode,
            getCustomModuleImportNode,
            getTestingLibraryImportName,
            getCustomModuleImportName,
            isTestingLibraryImported,
            isValidFilename,
            isGetByQuery,
            isQueryByQuery,
            isFindByQuery,
            isSyncQuery,
            isAsyncQuery,
            isCustomQuery,
            isAsyncUtil,
            isFireEventMethod,
            isPresenceAssert,
            isAbsenceAssert,
            canReportErrors,
            findImportedUtilSpecifier,
            isNodeComingFromTestingLibrary,
        };
        const detectionInstructions = {
            ImportDeclaration(node) {
                if (!importedTestingLibraryNode &&
                    /testing-library/g.test(node.source.value)) {
                    importedTestingLibraryNode = node;
                }
                if (!importedCustomModuleNode &&
                    String(node.source.value).endsWith(customModule)) {
                    importedCustomModuleNode = node;
                }
            },
            [`CallExpression > Identifier[name="require"]`](node) {
                const callExpression = node.parent;
                const { arguments: args } = callExpression;
                if (!importedTestingLibraryNode &&
                    args.some((arg) => node_utils_1.isLiteral(arg) &&
                        typeof arg.value === 'string' &&
                        /testing-library/g.test(arg.value))) {
                    importedTestingLibraryNode = callExpression;
                }
                if (!importedCustomModuleNode &&
                    args.some((arg) => node_utils_1.isLiteral(arg) &&
                        typeof arg.value === 'string' &&
                        arg.value.endsWith(customModule))) {
                    importedCustomModuleNode = callExpression;
                }
            },
        };
        const ruleInstructions = ruleCreate(context, optionsWithDefault, helpers);
        const enhancedRuleInstructions = {};
        const allKeys = new Set(Object.keys(detectionInstructions).concat(Object.keys(ruleInstructions)));
        allKeys.forEach((instruction) => {
            enhancedRuleInstructions[instruction] = (node) => {
                if (instruction in detectionInstructions) {
                    detectionInstructions[instruction](node);
                }
                if (canReportErrors() && ruleInstructions[instruction]) {
                    return ruleInstructions[instruction](node);
                }
            };
        });
        return enhancedRuleInstructions;
    };
}
exports.detectTestingLibraryUtils = detectTestingLibraryUtils;
