"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'prefer-screen-queries';
const ALLOWED_RENDER_PROPERTIES_FOR_DESTRUCTURING = [
    'container',
    'baseElement',
];
const ALL_QUERIES_COMBINATIONS_REGEXP = utils_1.ALL_QUERIES_COMBINATIONS.join('|');
function usesContainerOrBaseElement(node) {
    const secondArgument = node.arguments[1];
    return (node_utils_1.isObjectExpression(secondArgument) &&
        secondArgument.properties.some((property) => node_utils_1.isProperty(property) &&
            experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
            ALLOWED_RENDER_PROPERTIES_FOR_DESTRUCTURING.includes(property.key.name)));
}
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: 'Suggest using screen while using queries',
            category: 'Best Practices',
            recommended: 'error',
        },
        messages: {
            preferScreenQueries: 'Use screen to query DOM elements, `screen.{{ name }}`',
        },
        fixable: null,
        schema: [],
    },
    defaultOptions: [],
    create(context) {
        function reportInvalidUsage(node) {
            context.report({
                node,
                messageId: 'preferScreenQueries',
                data: {
                    name: node.name,
                },
            });
        }
        const queriesRegex = new RegExp(ALL_QUERIES_COMBINATIONS_REGEXP);
        const queriesDestructuredInWithinDeclaration = [];
        const withinDeclaredVariables = [];
        return {
            VariableDeclarator(node) {
                if (!node_utils_1.isCallExpression(node.init) ||
                    !experimental_utils_1.ASTUtils.isIdentifier(node.init.callee)) {
                    return;
                }
                const isWithinFunction = node.init.callee.name === 'within';
                const usesRenderOptions = node.init.callee.name === 'render' &&
                    usesContainerOrBaseElement(node.init);
                if (!isWithinFunction && !usesRenderOptions) {
                    return;
                }
                if (node_utils_1.isObjectPattern(node.id)) {
                    const identifiers = node.id.properties
                        .filter((property) => node_utils_1.isProperty(property) &&
                        experimental_utils_1.ASTUtils.isIdentifier(property.key) &&
                        queriesRegex.test(property.key.name))
                        .map((property) => property.key.name);
                    queriesDestructuredInWithinDeclaration.push(...identifiers);
                    return;
                }
                if (experimental_utils_1.ASTUtils.isIdentifier(node.id)) {
                    withinDeclaredVariables.push(node.id.name);
                }
            },
            [`CallExpression > Identifier[name=/^${ALL_QUERIES_COMBINATIONS_REGEXP}$/]`](node) {
                if (!queriesDestructuredInWithinDeclaration.some((queryName) => queryName === node.name)) {
                    reportInvalidUsage(node);
                }
            },
            [`MemberExpression > Identifier[name=/^${ALL_QUERIES_COMBINATIONS_REGEXP}$/]`](node) {
                function isIdentifierAllowed(name) {
                    return ['screen', ...withinDeclaredVariables].includes(name);
                }
                if (experimental_utils_1.ASTUtils.isIdentifier(node) &&
                    node_utils_1.isMemberExpression(node.parent) &&
                    node_utils_1.isCallExpression(node.parent.object) &&
                    experimental_utils_1.ASTUtils.isIdentifier(node.parent.object.callee) &&
                    node.parent.object.callee.name !== 'within' &&
                    node.parent.object.callee.name === 'render' &&
                    !usesContainerOrBaseElement(node.parent.object)) {
                    reportInvalidUsage(node);
                    return;
                }
                if (node_utils_1.isMemberExpression(node.parent) &&
                    experimental_utils_1.ASTUtils.isIdentifier(node.parent.object) &&
                    !isIdentifierAllowed(node.parent.object.name)) {
                    reportInvalidUsage(node);
                }
            },
        };
    },
});
