"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RULE_NAME = void 0;
const experimental_utils_1 = require("@typescript-eslint/experimental-utils");
const utils_1 = require("../utils");
const node_utils_1 = require("../node-utils");
exports.RULE_NAME = 'render-result-naming-convention';
const ALLOWED_VAR_NAMES = ['view', 'utils'];
const ALLOWED_VAR_NAMES_TEXT = ALLOWED_VAR_NAMES.map((name) => `\`${name}\``).join(', ');
exports.default = experimental_utils_1.ESLintUtils.RuleCreator(utils_1.getDocsUrl)({
    name: exports.RULE_NAME,
    meta: {
        type: 'suggestion',
        docs: {
            description: 'Enforce a valid naming for return value from `render`',
            category: 'Best Practices',
            recommended: false,
        },
        messages: {
            renderResultNamingConvention: `\`{{ varName }}\` is not a recommended name for \`render\` returned value. Instead, you should destructure it, or call it using one of the valid choices: ${ALLOWED_VAR_NAMES_TEXT}`,
        },
        fixable: null,
        schema: [
            {
                type: 'object',
                properties: {
                    renderFunctions: {
                        type: 'array',
                    },
                },
            },
        ],
    },
    defaultOptions: [
        {
            renderFunctions: [],
        },
    ],
    create(context, [options]) {
        const { renderFunctions } = options;
        let renderAlias;
        let wildcardImportName;
        return {
            ImportDeclaration(node) {
                if (!utils_1.hasTestingLibraryImportModule(node)) {
                    return;
                }
                const renderImport = node.specifiers.find((node) => node_utils_1.isImportSpecifier(node) && node.imported.name === 'render');
                if (!renderImport) {
                    return;
                }
                renderAlias = renderImport.local.name;
            },
            'ImportDeclaration ImportNamespaceSpecifier'(node) {
                if (!utils_1.hasTestingLibraryImportModule(node.parent)) {
                    return;
                }
                wildcardImportName = node.local.name;
            },
            VariableDeclarator(node) {
                if (node_utils_1.isObjectPattern(node.id)) {
                    return;
                }
                const isValidRenderDeclarator = node_utils_1.isRenderVariableDeclarator(node, [
                    ...renderFunctions,
                    renderAlias,
                ]);
                const isValidWildcardImport = !!wildcardImportName;
                if (!isValidRenderDeclarator && !isValidWildcardImport) {
                    return;
                }
                const renderFunctionName = node_utils_1.isCallExpression(node.init) &&
                    experimental_utils_1.ASTUtils.isIdentifier(node.init.callee) &&
                    node.init.callee.name;
                const renderFunctionObjectName = node_utils_1.isCallExpression(node.init) &&
                    node_utils_1.isMemberExpression(node.init.callee) &&
                    experimental_utils_1.ASTUtils.isIdentifier(node.init.callee.property) &&
                    experimental_utils_1.ASTUtils.isIdentifier(node.init.callee.object) &&
                    node.init.callee.property.name === 'render' &&
                    node.init.callee.object.name;
                const isRenderAlias = !!renderAlias;
                const isCustomRender = renderFunctions.includes(renderFunctionName);
                const isWildCardRender = renderFunctionObjectName &&
                    renderFunctionObjectName === wildcardImportName;
                if (!isRenderAlias && !isCustomRender && !isWildCardRender) {
                    return;
                }
                const renderResultName = experimental_utils_1.ASTUtils.isIdentifier(node.id) && node.id.name;
                const isAllowedRenderResultName = ALLOWED_VAR_NAMES.includes(renderResultName);
                if (isAllowedRenderResultName) {
                    return;
                }
                context.report({
                    node,
                    messageId: 'renderResultNamingConvention',
                    data: {
                        varName: renderResultName,
                    },
                });
            },
        };
    },
});
